/*
 *  Copyright 2008 Mikhail Kryshen
 *
 *  This file is part of Charamega.
 *
 *  Charamega is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Charamega is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Charamega.  If not, see <http://www.gnu.org/licenses/>.
 */

package kryshen.charamega;

import javafx.ui.*;
import javafx.ui.canvas.*;
import javafx.ui.filter.*;
import javafx.animation.*;
import java.lang.Math;
import java.lang.System;
import java.lang.Character;
import com.sun.javafx.runtime.PointerFactory;

/**
 * Card view.
 */
class Card extends CompositeNode {
    private attribute pf = PointerFactory {};
    
    attribute field:Field;
    attribute model:CardModel;      
      
    attribute openAnimation = Timeline {
        toggle: true
        keyFrames: [
            KeyFrame {
                time: 0s
                values: [
                KeyValue {
                    target: bind pf.make(scale.x)
                    value: 1.0
                    interpolate: Interpolator.EASEBOTH
                },
                KeyValue {
                    target: bind pf.make(char.visible)
                    value: false
                    interpolate: Interpolator.DISCRETE
                },
                KeyValue {
                    target: bind pf.make(rect.fill)
                    value: Style.CARD_BACK_COLOR
                    interpolate: Interpolator.DISCRETE
                }
                ]
            },
            KeyFrame {
                time: 100ms
                values: [
                KeyValue {
                    target: bind pf.make(scale.x)
                    value: 0.0
                    interpolate: Interpolator.EASEBOTH
                },
                KeyValue {
                    target: bind pf.make(char.visible)
                    value: true
                    interpolate: Interpolator.DISCRETE
                },
                KeyValue {
                    target: bind pf.make(rect.fill)
                    value: Style.CARD_FACE_COLOR
                    interpolate: Interpolator.DISCRETE
                }
                ]
            },
            KeyFrame {
                time: 200ms
                values:
                KeyValue {
                    target: bind pf.make(scale.x)
                    value: 1.0
                    interpolate: Interpolator.EASEBOTH
                }
            }
        ]
    };
    
    attribute solveAnimation = Timeline {
        toggle: true
        keyFrames: [
            KeyFrame {
                time: 0s
                values: KeyValue {
                    target: bind pf.make(opacity);
                    value: 1.0
                    interpolate: Interpolator.EASEBOTH
                }
            },
            KeyFrame {
                time: 200ms
                values: KeyValue {
                    target: bind pf.make(opacity);
                    value: 0.0
                    interpolate: Interpolator.EASEBOTH
                }
            }
        ]
    };
    
    attribute opened = bind model.opened on replace o = n {
        if (o <> n) {
            openAnimation.start();
        }
    };
    
    attribute solved = bind model.solved on replace o = n {
        if (o <> n) {
            solveAnimation.start();
        }
    };
        
    private attribute rect:Rect = Rect {
        fill: Style.CARD_BACK_COLOR
        stroke: Style.CARD_BORDER_COLOR
        width: bind model.field.cardSize
        height: bind model.field.cardSize
    };        

    attribute font = bind Font {
        face: FontFace {
            url: getClass().getResource("DejaVuSans.ttf").toString()
        }
        size: model.field.cardSize * 3 / 4 as Integer
    }
    
    private attribute char:Text = Text {
        valign: VerticalAlignment.CENTER
        halign: HorizontalAlignment.CENTER
        
        x: bind model.field.cardSize / 2
        y: bind model.field.cardSize / 2
        
        visible: false
        
        font: bind font        
        fill: Style.CARD_CHAR_COLOR
        
        content: bind new String(Character.toChars(model.value));
    }
    
    attribute scale = Scale {
        x: 1.0
        y: 1.0
    }
    
    public function composeNode():Node {
        return Group {
            transform: bind [
                Translate { 
                    x: model.x + (1 - scale.x) * (model.field.cardSize / 2.0)
                    y: model.y + (1 - scale.y) * (model.field.cardSize / 2.0)
                },
                scale,
                Rotate { 
                    angle: model.angle
                    cx: bind model.field.cardSize / 2
                    cy: bind model.field.cardSize / 2
                }                
            ]
            
            cursor: Cursor.HAND            
            
            content: [rect, char]
            
            onMouseEntered: function(e) {
                rect.stroke = Style.CARD_SELECTED_BORDER_COLOR;
            }
            
            onMouseExited: function(e) {
                rect.stroke = Style.CARD_BORDER_COLOR;
            }
            
            onMouseClicked: function(e) {
                //System.err.println(model.value);
                field.interruptWait();
                
                if (model.open()) {
                    field.checkOpened();
                }
            }
        }
    }
}
