/*
 *  Copyright 2008 Mikhail Kryshen
 *
 *  This file is part of Charamega.
 *
 *  Charamega is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Charamega is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Charamega.  If not, see <http://www.gnu.org/licenses/>.
 */

package kryshen.charamega;

import javafx.lang.*;
import javafx.ui.*;
import javafx.ui.canvas.*;
import javafx.ui.filter.*;

import java.awt.Desktop;
import java.net.URI;

public class Game {
    static attribute TITLE = "Charamega";
    static attribute AUTHOR = "Mikhail Kryshen";
    static attribute VERSION = "0.1";
    static attribute HOMEPAGE = "http://kryshen.pp.ru/games/";

    private attribute timerFormat = 
        new java.text.SimpleDateFormat("mm:ss");

    attribute applet:Applet;

    attribute field:Field = Field {
        width: bind canvas.width
        height: bind canvas.height
    };

    private attribute canvas:Canvas = Canvas {    
        background: Style.FIELD_BACKGROUND
        content: field
        visible: bind field.model.status <> FieldModel.STATUS_STOPPED
    };

    private attribute about:Label = Label {
        border: Style.ABOUT_BORDER
        background: Style.FIELD_BACKGROUND
        text: "Created by {AUTHOR}\nVersion: {VERSION}"
        visible: bind field.model.status == FieldModel.STATUS_STOPPED   
    };
   
    attribute mainPanel = BorderPanel {
        opaque: true

        border: Style.MAIN_BORDER

        center: BorderPanel {
            border: Style.FIELD_BORDER
            center: StackPanel {
                content: [about, canvas]
            }
        }

        top:
        GroupPanel {        
            var row = Row { alignment: Alignment.CENTER }
            var column1 = Column { resizable: true }
            var column2 = Column { alignment: Alignment.TRAILING }    

            border: Style.TITLE_BORDER

            autoCreateContainerGaps: false
            rows: row
            columns: [column1, column2]

            content: [
            SimpleLabel {
                font: Style.TITLE_FONT
                text: TITLE
                row: row
                column: column1
                },
            SimpleLabel {
                text: "<html><a href=\"{HOMEPAGE}\">{HOMEPAGE}</a></html>"
                cursor: Cursor.HAND
                row: row
                column: column2

                onMouseClicked: function(e) { 
                    if (applet <> null) {
                        applet.showDocument(HOMEPAGE);                        
                    } else {
                        Desktop.getDesktop().browse(new URI(HOMEPAGE));
                    }
                }
            }]
        }

        bottom:
        Box {
            border: Style.STATUS_BORDER

            orientation: Orientation.HORIZONTAL
            content: [
            Button {
                text: "New Game"
                action: function() {                
                    field.model.start(40, field.width, field.height)
                }
            },
            FlowPanel {
                alignment: Alignment.TRAILING
                vgap: 0
                hgap: 15

                content: [
                SimpleLabel {
                    text: bind 
                    if (field.model.status == FieldModel.STATUS_WIN)
                        "YOU WIN"
                    else if (field.model.status == FieldModel.STATUS_LOSE)
                        "GAME OVER"
                    else
                        ""

                    foreground: bind 
                    if (field.model.status == FieldModel.STATUS_WIN)
                        Style.WIN_STATUS_COLOR
                    else 
                        Style.LOSE_STATUS_COLOR

                    font: Style.STATUS_FONT
                },
                SimpleLabel {
                    text: bind "Pairs left: ".concat(
                        (field.model.pairs - field.model.cleared).toString())
                    font: Style.STATUS_FONT
                },
                SimpleLabel {
                    text: bind "Mismatches: {field.model.mismatches}/".concat(
                               "{field.model.mismatchLimit}");
                    font: Style.STATUS_FONT
                },
                SimpleLabel {
                    text: bind 
                        "Time left: ".concat(
                        timerFormat.format(
                            Duration {
                                millis: field.model.timeLimit - field.model.time
                            }.toDate()))
                    font: Style.STATUS_FONT
                }]
            }]
        }
    };
}

var game = Game {}

Frame {
    title: TITLE
    width: 700
    height: 500
    content: game.mainPanel
    visible: true

    onClose: game.field.model.stop
}
